
// Simple demo of memory allocation errors in C
// Warning: use with care (== NOT on servers, ...)

// Compile using: gcc -std=c11  // and optionally try -m32
// 1) Run without commandline argument
// 2) Run with any commandline argument to really use the allocated memory
//    SSD Warning: Swap will be used (use swapoff OR ulimit -v)
// Linux: Note the number of allocations (due to "optimistic" allocation)
//        Can be switched off by: echo 2 > /proc/sys/vm/overcommit_memory

#include <stdio.h>      // printf
#include <stdlib.h>     // malloc
#include <string.h>     // memset

#define SIZE (1ULL<<30)  // 1GiB

int main(int argc, char * argv[])
{
    for(int i=0; ; ++i) {
        void *p = malloc(SIZE);
        if(p==NULL) {
            printf("%d allocations of %lld Bytes\n", i, SIZE);
            break;
        }
        if(argc>1)
            memset (p, 0, SIZE);
    }
    // memory leaks
    return 1;
}

/* experimental results (PC 8GiB RAM + 6GiB swap, Linux, no ulimit)

$ gcc -std=c11 ijc-alloc-test.c
$ ./a.out
131070 allocations of 1073741824 Bytes
$ ./a.out 1
12 allocations of 1073741824 Bytes

$ gcc -std=c11 ijc-alloc-test.c -m32
$ ./a.out
3 allocations of 1073741824 Bytes
$ ./a.out 1
3 allocations of 1073741824 Bytes

*/
