
// Test of quad math (double double) in GCC
// Link with libquadmath:
// gcc quad-test.c -lquadmath 

#include <quadmath.h>
#include <stdlib.h>
#include <stdio.h>

const int precision = 35;       // decimal digits to print

// print quad number
void printq(__float128 r) {
    char buf[128];
#if 1
    int n = quadmath_snprintf(buf, sizeof(buf), "%+-#.*Qf", precision, r);
#else
    int n = quadmath_snprintf(buf, sizeof buf, "%Qa", r); // hex
#endif
    if ((size_t) n < sizeof(buf))
        printf("%s\n", buf);
    else
        printf("printq: size error\n");
}

int main()
{
    __float128 r;
    printf("  .         1         2         3         4\n");
    printf("  .1234567890123456789012345678901234567890\n");
    printf("  .=========+=========+=========+=========+\n");

    printf("FLT128_EPSILON = \n");
    printq(FLT128_EPSILON);

    printf("sqrtq(2.0q) = \n");
    r = 2.0q;           // quad literal
    r = sqrtq(r);
    printq(r);
    printf(" 1.4142135623730950488016887242096980785696...exact\n");

    printf("M_PIq = \n");
    r = M_PIq;
    printq(r);
    printf(" 3.1415926535897932384626433832795028841968...exact\n");

    printf("sinq(M_PIq) = \n");
    r = sinq(M_PIq);
    printq(r);
    if(r!=0) printf("Warning: sinq(M_PIq)!=0\n");

    printf("expq(1) = \n");
    r = expq(1);
    printq(r);

    const char * s = "0.1234567890123456789012345678901234567890";
    printf("s = \n");
    printf(" %s\n",s);

    printf("strtoflt128(s) = \n");
    r = strtoflt128(s, NULL);
    printq(r);

    const __float128 r2 = 0.1234567890123456789012345678901234567890q;
    if(r!=r2) printf("Literal conversion differs\n");

    return 0;
}
